@@ -61,128 +61,128 @@ pub fn cli() -> clap::App<'static, 'static> {
                         )
                         .subcommand(SubCommand::with_name("data").about("NDEF data")),
                 )
-                // .subcommand(
-                //     SubCommand::with_name("oath")
-                //         .about("OATH app")
-                //         .setting(AppSettings::SubcommandRequiredElseHelp)
-                //         .setting(AppSettings::InferSubcommands)
-                //         .subcommand(SubCommand::with_name("aid").about("Prints the application's AID"))
-                //         .subcommand(SubCommand::with_name("register")
-                //             .about("Registers an OATH secret")
-                //             // FYI, imf = OATH initial moving factor
-                //             .arg(Arg::with_name("algorithm")
-                //                  .long("algorithm")
-                //                  .short("a")
-                //                  .help("hash algorithm to use in OTP generation")
-                //                  .value_name("DIGEST")
-                //                  .default_value("SHA1")
-                //                  .possible_values(&["SHA1", "SHA256"])
-                //                  .case_insensitive(true)
-                //              )
-                //             .arg(Arg::with_name("counter")
-                //                  .long("counter")
-                //                  .short("c")
-                //                  .help("(only HOTP) initial counter to use for HOTPs")
-                //                  .value_name("INITIAL-COUNTER")
-                //                  .default_value("0")
-                //                  .required_if("kind", "HOTP")
-                //              )
-                //             .arg(Arg::with_name("digits")
-                //                  .long("digits")
-                //                  .short("d")
-                //                  .help("number of digits to use for the OATH OTP values")
-                //                  .value_name("ISSUER")
-                //                  .default_value("6")
-                //                  .possible_values(&["6", "7", "8"])
-                //              )
-                //             .arg(Arg::with_name("issuer")
-                //                  .long("issuer")
-                //                  .short("i")
-                //                  .help("(optional) issuer to use for the OATH credential, e.g., example.com")
-                //                  .value_name("ISSUER")
-                //              )
-                //             .arg(Arg::with_name("kind")
-                //                  .long("kind")
-                //                  .short("k")
-                //                  // for compatibility with `ykman`
-                //                  .aliases(&["o", "oath-type"])
-                //                  .help("kind of OATH credential to register")
-                //                  .value_name("KIND")
-                //                  .default_value("TOTP")
-                //                  .case_insensitive(true)
-                //                  .possible_values(&["HOTP", "TOTP"])
-                //              )
-                //             .arg(Arg::with_name("period")
-                //                  .long("period")
-                //                  .short("p")
-                //                  .help("(only TOTP) period in seconds for which a TOTP should be valid")
-                //                  .value_name("PERIOD")
-                //                  .default_value("30")
-                //                  .required_if("kind", "TOTP")
-                //              )
-                //             .arg(Arg::with_name("label")
-                //                  .help("label to use for the OATH secret, e.g. alice@trussed.dev")
-                //                  .value_name("LABEL")
-                //                  .required(true)
-                //              )
-                //             .arg(Arg::with_name("secret")
-                //                  .help("the actual TOTP seed, e.g. JBSWY3DPEHPK3PXPJBSWY3DPEHPK3PXP")
-                //                  .value_name("SECRET")
-                //                  .required(true)
-                //              )
-                //             .arg(Arg::with_name("sha1")
-                //                  .long("sha1")
-                //                  .help("use SHA1 hash algorithm in OTP generation")
-                //                  .conflicts_with_all(&["sha256", "algorithm"])
-                //              )
-                //             .arg(Arg::with_name("sha256")
-                //                  .long("sha256")
-                //                  .help("use SHA256 hash algorithm in OTP generation")
-                //                  .conflicts_with_all(&["sha1", "algorithm"])
-                //              )
-                //             .arg(Arg::with_name("hotp")
-                //                  .long("hotp")
-                //                  .help("generate HOTP flavor of OATH credentials.")
-                //                  .conflicts_with_all(&["totp", "kind"])
-                //              )
-                //             .arg(Arg::with_name("totp")
-                //                  .long("totp")
-                //                  .help("generate TOTP flavor of OATH credentials.")
-                //                  .conflicts_with_all(&["hotp", "kind"])
-                //              )
-                //         )
-                //         .subcommand(SubCommand::with_name("totp")
-                //             .about("Generates a TOTP from a previously registered credential")
-                //             .arg(Arg::with_name("TIMESTAMP")
-                //                  .short("t")
-                //                  .long("timestamp")
-                //                  .hidden(true)
-                //                  .help("timestamp to use to generate the OTP, as seconds since the UNIX epoch")
-                //                  .value_name("TIMESTAMP")
-                //                  .required(false)
-                //              )
-                //             .arg(Arg::with_name("label")
-                //                  .help("label of the TOTP credential to use, e.g. alice@trussed.dev")
-                //                  .value_name("LABEL")
-                //                  .required(true)
-                //              )
-                //         )
-                //         .subcommand(SubCommand::with_name("delete")
-                //             .about("Deletes a previously registered credential")
-                //             .arg(Arg::with_name("label")
-                //                  .help("label of the TOTP credential to delete, e.g. alice@trussed.dev")
-                //                  .value_name("LABEL")
-                //                  .required(true)
-                //              )
-                //         )
-                //         .subcommand(SubCommand::with_name("list")
-                //             .about("Lists credentials stored on the device")
-                //             .visible_alias("ls")
-                //         )
-                //         .subcommand(SubCommand::with_name("reset")
-                //             .about("Reset device, deleting all credentials")
-                //         )
-                // )
+                .subcommand(
+                    SubCommand::with_name("oath")
+                        .about("OATH app")
+                        .setting(AppSettings::SubcommandRequiredElseHelp)
+                        .setting(AppSettings::InferSubcommands)
+                        .subcommand(SubCommand::with_name("aid").about("Prints the application's AID"))
+                        .subcommand(SubCommand::with_name("register")
+                            .about("Registers an OATH secret")
+                            // FYI, imf = OATH initial moving factor
+                            .arg(Arg::with_name("algorithm")
+                                 .long("algorithm")
+                                 .short("a")
+                                 .help("hash algorithm to use in OTP generation")
+                                 .value_name("DIGEST")
+                                 .default_value("SHA1")
+                                 .possible_values(&["SHA1", "SHA256"])
+                                 .case_insensitive(true)
+                             )
+                            .arg(Arg::with_name("counter")
+                                 .long("counter")
+                                 .short("c")
+                                 .help("(only HOTP) initial counter to use for HOTPs")
+                                 .value_name("INITIAL-COUNTER")
+                                 .default_value("0")
+                                 .required_if("kind", "HOTP")
+                             )
+                            .arg(Arg::with_name("digits")
+                                 .long("digits")
+                                 .short("d")
+                                 .help("number of digits to use for the OATH OTP values")
+                                 .value_name("ISSUER")
+                                 .default_value("6")
+                                 .possible_values(&["6", "7", "8"])
+                             )
+                            .arg(Arg::with_name("issuer")
+                                 .long("issuer")
+                                 .short("i")
+                                 .help("(optional) issuer to use for the OATH credential, e.g., example.com")
+                                 .value_name("ISSUER")
+                             )
+                            .arg(Arg::with_name("kind")
+                                 .long("kind")
+                                 .short("k")
+                                 // for compatibility with `ykman`
+                                 .aliases(&["o", "oath-type"])
+                                 .help("kind of OATH credential to register")
+                                 .value_name("KIND")
+                                 .default_value("TOTP")
+                                 .case_insensitive(true)
+                                 .possible_values(&["HOTP", "TOTP"])
+                             )
+                            .arg(Arg::with_name("period")
+                                 .long("period")
+                                 .short("p")
+                                 .help("(only TOTP) period in seconds for which a TOTP should be valid")
+                                 .value_name("PERIOD")
+                                 .default_value("30")
+                                 .required_if("kind", "TOTP")
+                             )
+                            .arg(Arg::with_name("label")
+                                 .help("label to use for the OATH secret, e.g. alice@trussed.dev")
+                                 .value_name("LABEL")
+                                 .required(true)
+                             )
+                            .arg(Arg::with_name("secret")
+                                 .help("the actual TOTP seed, e.g. JBSWY3DPEHPK3PXPJBSWY3DPEHPK3PXP")
+                                 .value_name("SECRET")
+                                 .required(true)
+                             )
+                            .arg(Arg::with_name("sha1")
+                                 .long("sha1")
+                                 .help("use SHA1 hash algorithm in OTP generation")
+                                 .conflicts_with_all(&["sha256", "algorithm"])
+                             )
+                            .arg(Arg::with_name("sha256")
+                                 .long("sha256")
+                                 .help("use SHA256 hash algorithm in OTP generation")
+                                 .conflicts_with_all(&["sha1", "algorithm"])
+                             )
+                            .arg(Arg::with_name("hotp")
+                                 .long("hotp")
+                                 .help("generate HOTP flavor of OATH credentials.")
+                                 .conflicts_with_all(&["totp", "kind"])
+                             )
+                            .arg(Arg::with_name("totp")
+                                 .long("totp")
+                                 .help("generate TOTP flavor of OATH credentials.")
+                                 .conflicts_with_all(&["hotp", "kind"])
+                             )
+                        )
+                        .subcommand(SubCommand::with_name("totp")
+                            .about("Generates a TOTP from a previously registered credential")
+                            .arg(Arg::with_name("TIMESTAMP")
+                                 .short("t")
+                                 .long("timestamp")
+                                 .hidden(true)
+                                 .help("timestamp to use to generate the OTP, as seconds since the UNIX epoch")
+                                 .value_name("TIMESTAMP")
+                                 .required(false)
+                             )
+                            .arg(Arg::with_name("label")
+                                 .help("label of the TOTP credential to use, e.g. alice@trussed.dev")
+                                 .value_name("LABEL")
+                                 .required(true)
+                             )
+                        )
+                        .subcommand(SubCommand::with_name("delete")
+                            .about("Deletes a previously registered credential")
+                            .arg(Arg::with_name("label")
+                                 .help("label of the TOTP credential to delete, e.g. alice@trussed.dev")
+                                 .value_name("LABEL")
+                                 .required(true)
+                             )
+                        )
+                        .subcommand(SubCommand::with_name("list")
+                            .about("Lists credentials stored on the device")
+                            .visible_alias("ls")
+                        )
+                        .subcommand(SubCommand::with_name("reset")
+                            .about("Reset device, deleting all credentials")
+                        )
+                )
                 .subcommand(
                     SubCommand::with_name("provisioner")
                         .about("Provisioner app")
@@ -275,24 +275,46 @@ pub fn cli() -> clap::App<'static, 'static> {
         )
         // dev PKI
         .subcommand(
-            SubCommand::with_name("dev-pki")
-                .about("PKI for development")
+            SubCommand::with_name("pki")
+                .about("PKI-related")
                 .setting(AppSettings::SubcommandRequiredElseHelp)
                 .setting(AppSettings::InferSubcommands)
                 .subcommand(
-                    SubCommand::with_name("fido")
-                        .about("Generates a self-signed FIDO batch attestation cert+key")
-                        .arg(
-                            Arg::with_name("KEY")
-                                .help("output file, for private P256 key in binary format")
-                                .required(true),
+                    SubCommand::with_name("ca")
+                        .about("CA-related")
+                        .setting(AppSettings::SubcommandRequiredElseHelp)
+                        .setting(AppSettings::InferSubcommands)
+                        .subcommand(
+                            SubCommand::with_name("fetch-certificate")
+                                .about("Fetch one of the well-known Solo 2 PKI certificates in DER format")
+                                .arg(
+                                    Arg::with_name("AUTHORITY")
+                                        .help("name of authority, e.g. R1, T1, S3, etc.")
+                                        .required(true),
+                                )
                         )
-                        .arg(
-                            Arg::with_name("CERT")
-                                .help("output file, for self-signed certificate in DER format")
-                                .required(true),
+                )
+                .subcommand(
+                    SubCommand::with_name("dev")
+                        .about("PKI for development")
+                        .setting(AppSettings::SubcommandRequiredElseHelp)
+                        .setting(AppSettings::InferSubcommands)
+                        .subcommand(
+                            SubCommand::with_name("fido")
+                                .about("Generates a self-signed FIDO batch attestation cert+key")
+                                .arg(
+                                    Arg::with_name("KEY")
+                                        .help("output file, for private P256 key in binary format")
+                                        .required(true),
+                                )
+                                .arg(
+                                    Arg::with_name("CERT")
+                                        .help("output file, for self-signed certificate in DER format")
+                                        .required(true),
+                                ),
                         ),
-                ),
+                    ),
+
         )
         // inherited from lpc55-host
         .subcommand(
@@ -333,14 +355,14 @@ pub fn cli() -> clap::App<'static, 'static> {
                     Arg::with_name("yes")
                         .short("y")
                         .long("yes")
-                        .help("Proceed with major updates without prompt.")
+                        .help("DANGER! Proceed with major updates without prompt.")
                         .required(false),
                 )
                 .arg(
                     Arg::with_name("all")
                         .short("a")
                         .long("all")
-                        .help("Update all connect Solo devices.")
+                        .help("Update all connected SoloKeys Solo 2 devices.")
                         .required(false),
                 )
                 .arg(
